'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Storage = exports.Response = exports.Controller = exports.Route = undefined;

var _response = require('./request/response');

var _response2 = _interopRequireDefault(_response);

var _route2 = require('./request/route');

var _route3 = _interopRequireDefault(_route2);

var _controller = require('./request/controller');

var _controller2 = _interopRequireDefault(_controller);

var _stringUtil = require('./stringUtil');

var _storeContainer = require('./storage/storeContainer');

var _storeContainer2 = _interopRequireDefault(_storeContainer);

var _store = require('./storage/store');

var _store2 = _interopRequireDefault(_store);

var _storable = require('./storage/storable');

var _storable2 = _interopRequireDefault(_storable);

var _storableLink = require('./storage/storableLink');

var _storableLink2 = _interopRequireDefault(_storableLink);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createError = function createError(obj) {
  return obj;
};

/**
 * Create a faussaire instance
 *
 * @returns {Object}
 */
var create = function create() {

  var _routes = [];
  var _notFoundResponse = (0, _response2.default)({
    data: {},
    status: 404,
    statusText: "Route not found.",
    headers: {}
  });

  var faussaire = {
    storage: _storeContainer2.default.createStoreContainer(),

    /**
     * Add a route to faussaire
     * A route is represented by a template and the HTTP methods.
     *
     * A controller is called once the associated route match.
     * The authenticate method allows to return a token to be passed in the option
     * object for the next run call.
     *
     * The run function is the only one to be able to return a response, which is an object
     * corresponding to the response object definition (see response.js)
     */
    route: function route(_route) {
      _routes.push(_route);
      return faussaire;
    },

    /**
     * Fetch the data synchronously.
     * @param url
     * @param method
     * @param requestBody
     * @returns Promise
     */
    fetch: function fetch(url, method) {
      var requestBody = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      return new Promise(function (accept, reject) {

        // First finding the route and checking the method
        var matchingRoute = _routes.find(function (route) {
          return (0, _stringUtil.isMatching)(route.template, url) && route.methods.indexOf(method.toUpperCase()) >= 0;
        });

        // If no route is found, throw a notFound error.
        if (!matchingRoute) {
          reject(createError({
            response: _notFoundResponse
          }));

          return;
        }

        var query = Object.assign({}, (0, _stringUtil.extractURLArgs)(url), requestBody.params),
            request = [],
            route = (0, _stringUtil.extractRouteParameters)(matchingRoute.template, url);

        // In case of anything but a GET request, we fill the request variable with the body's data
        if (method !== "GET") {
          request = requestBody.data;
        }

        // params to be passed to runners
        var params = {
          query: query,
          request: request,
          route: route
        };

        // Object holding data about the process
        var options = {
          method: method,
          token: undefined
        };

        // Authenticating if required
        if (typeof matchingRoute.controller.authenticate === 'function') {
          var token = matchingRoute.controller.authenticate(params, options);

          // If the token returned is a falsy value, then authentication failed.
          if (token) {
            options.token = token;
          }
        }

        var response = matchingRoute.controller.run(params, options);
        if (response.status >= 400) {
          reject(createError({
            response: response
          }));

          return;
        }

        return accept(response);
      });
    },

    /**
     * Set custom error when not found
     * @param response
     */
    setNotFoundResponse: function setNotFoundResponse(response) {
      _notFoundResponse = response;
    }
  };

  return faussaire;
};

exports.default = create();
var Route = exports.Route = _route3.default;
var Controller = exports.Controller = _controller2.default;
var Response = exports.Response = _response2.default;
var Storage = exports.Storage = {
  storeContainerFactory: _storeContainer2.default,
  storeFactory: _store2.default,
  storableFactory: _storable2.default,
  storableLinkFactory: _storableLink2.default
};