'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _storableLink = require('./storableLink');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var storableToScalarObject = function storableToScalarObject(data) {
  var nextData = {};

  // We process the object in case of storableLink to manage
  Object.keys(data).forEach(function (key) {
    if ((0, _storableLink.isStorableLink)(data[key])) {
      var storable = data[key].getStorable();

      if (Array.isArray(storable)) {
        nextData[key] = storable.map(function (item) {
          return storableToScalarObject(item.getData());
        });
      } else {
        nextData[key] = storableToScalarObject(storable.getData());
      }
    } else {
      nextData[key] = data[key];
    }
  });

  return nextData;
};

/**
 * Create a StoreContainer
 * @returns {StoreContainerType}
 */
var createStoreContainer = function createStoreContainer() {
  var stores = [];

  var storeContainer = {
    /**
     * Return a store given his name
     * @param name
     * @returns {StoreType}
     */
    getStore: function getStore(name) {
      return stores.find(function (entity) {
        return entity.name === name;
      });
    },

    /**
     * Add a store to the container
     * @param store
     * @returns {StoreContainerType}
     */
    addStore: function addStore(store) {
      stores = [].concat(_toConsumableArray(stores), [store]);

      return storeContainer;
    },

    /**
     * Return the array of stores
     * @returns {Array<StoreType>}
     */
    all: function all() {
      return stores;
    },

    /**
     * Reset all the stores to their initial state
     * @returns {StoreContainerType}
     */
    resetAll: function resetAll() {
      [].slice.call(stores).forEach(function (store) {
        store.reset();
      });
      return storeContainer;
    },

    /**
     * Assemble an entity, returns the scalar version of the data
     * @param entity
     * @param config
     * @returns {{}}
     */
    assemble: function assemble(entity) {
      var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        schema: []
      };

      var data = Object.assign({}, entity.getData());

      // If there's schema
      if (config.hasOwnProperty("schema") && Array.isArray(config.schema) && config.schema.length) {
        (function () {

          // We remove all the keys we don't want
          var schema = config.schema;
          Object.keys(data).forEach(function (key) {
            if (schema.indexOf(key) < 0) {
              delete data[key];
            }
          });
        })();
      }

      return storableToScalarObject(data);
    },

    /**
     * Assemble all the keys of the object
     * This is used when you want to assemble an object composed of multiple StorableType combined with
     * scalar values
     *
     * @param object
     * @param config
     * @returns {{}}
     */
    assembleAll: function assembleAll(object) {
      var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
        schema: []
      };

      var data = Object.assign({}, object);

      // If there's schema
      if (config.hasOwnProperty("schema") && Array.isArray(config.schema) && config.schema.length) {
        (function () {

          // We remove all the keys we don't want
          var schema = config.schema;
          Object.keys(data).forEach(function (key) {
            if (schema.indexOf(key) < 0) {
              delete data[key];
            }
          });
        })();
      }

      var nextData = {};

      Object.keys(data).forEach(function (key) {
        if (data[key].getData) {
          nextData[key] = storeContainer.assemble(data[key]);
        } else {
          nextData[key] = data[key];
        }
      });

      return nextData;
    }
  };

  return storeContainer;
};

exports.default = {
  createStoreContainer: createStoreContainer
};

/*
 // Return only the wanted keys
 Object.keys(data).forEach((key: string) => {
 if(schema.indexOf(key) >= 0){
 if(isStorableLink(data[key])){
 let storable = data[key].getStorable();

 if(Array.isArray(storable)){
 nextData[key] = storable.map((item: StorableType) => item.getData());
 } else {

 }
 } else {
 nextData[key] = data[key];
 }
 }
 });

 return nextData;
 */